package ch.rakudave.jnetmap.plugins;

import java.io.File;
import java.util.Arrays;

import javax.swing.Icon;

import ch.rakudave.jnetmap.controller.Scheduler;
import ch.rakudave.jnetmap.model.device.Device;
import ch.rakudave.jnetmap.util.IO;
import ch.rakudave.jnetmap.util.Icons;
import ch.rakudave.jnetmap.util.SwingHelper;
import ch.rakudave.jnetmap.util.logging.Logger;
import ch.rakudave.jnetmap.view.preferences.PreferencePanel;

public class RightClickExec implements RightClickAction {
	private String name, command, args;
	
	private RightClickExec() {}
	
	public RightClickExec(String name, String command, String args) {
		this();
		this.name = name;
		this.command = command;
		this.args = args;
	}
	
	/**
	 * This command will be executed when a user right-clicks a device and selects this item.
	 * @param d that device that was right-clicked, use this to extract the address etc.
	 */
	public void execute(final Device d) {
		Scheduler.execute(new Runnable() {
			@Override
			public void run() {
				Process process = null;
				try {
					String[] commandAndArgs = IO.splitCommandArgs(command, fillArgs(args, d));
					Logger.trace("executing script: " + Arrays.toString(commandAndArgs));
					process = Runtime.getRuntime().exec(commandAndArgs);
					process.waitFor();
				} catch (Exception ex) {
					Logger.warn("Unable to execute script", ex);
				} finally {
					if (process != null) process.destroy();
				}
			}
		});
	}

	private String fillArgs(String args, Device d) {
		if (args == null || "".equals(args)) return "";
		String result = args.replaceAll("%s", d.getStatus().toString())
				.replaceAll("%n", d.getName())
				.replaceAll("%t", d.getType().toString())
				.replaceAll("%d", d.getDesctription())
				.replaceAll("%l", d.getLocation())
				.replaceAll("%m", d.getModel())
				.replaceAll("%v", d.getVendor())
				.replaceAll("%i", new File(IO.userDir, Icons.currentTheme()+"/"+d.getType().toString().toLowerCase()+".png").getAbsolutePath());
		if (args.contains("%a")) result = result.replaceAll("%a", SwingHelper.interfaceSelector(d, false)+"");
		if (args.contains("%h")) result = result.replaceAll("%h", SwingHelper.interfaceSelector(d, true)+"");
		return result;
	}

	@Override
	public String getPluginName() {
		return name;
	}

	@Override
	public String getAuthor() {
		return "";
	}

	@Override
	public String getDescription() {
		return "";
	}

	@Override
	public Icon getIcon() {
		return Icons.get("run");
	}

	@Override
	public boolean hasSettings() {
		return false;
	}

	@Override
	public PreferencePanel getSettingsPanel() {
		return null;
	}

	@Override
	public String toString() {
		return name+": "+command+" "+args;
	}
}
